export P=proj
export V=6.2.1
export B=2
export MAINTAINER=JuergenFischer

savelog() {
	local f=$1
	/bin/rm -f $f.9
	for i in $(seq 8 -1 0); do [ -f "$f.$i" ] && mv $f.$i $f.$(( i + 1 )); done
	[ -f $f ] && mv $f $f.0
}

savelog package.log
exec > >(tee package.log) 2>&1
date

set -e

[ -f proj-$V.tar.gz ] || wget https://download.osgeo.org/proj/proj-$V.tar.gz
[ -f patched ] || patch -p1 -d.. <0001-normalizeForVisualization-and-other-methods-applying.patch >patched

for i in 1.8 north-america-1.2 europe-1.4 oceania-1.0 world-1.0; do
	[ -f proj-datumgrid-$i.tar.gz ] || wget https://download.osgeo.org/proj/proj-datumgrid-$i.tar.gz
done

[ -f ../CMakeLists.txt ] || tar -C .. -xzf proj-$V.tar.gz --xform s,proj-${V%RC*},.,

if ! [ -d "$OSGEO4W_REP" ]; then
        echo no repository
        exit 1
fi

fetchenv() {
        local IFS
        IFS=
        batch=$1
        shift

        if ! [ -f "$batch" ]; then
                echo "$batch not found"
                return 1
        fi

        srcenv=$(mktemp /tmp/srcenv.XXXXXXXXXX)
        dstenv=$(mktemp /tmp/dstenv.XXXXXXXXXX)
        diffenv=$(mktemp /tmp/diffenv.XXXXXXXXXX)
        envdiff=$(mktemp /tmp/envdiff.XXXXXXXXXX)
        cmd /c set >$srcenv
        cmd /c call $(cygpath -w $batch) "$@" \>nul 2\>nul \& set >$dstenv
        /usr/bin/cat <<EOF >$envdiff
/^+++/d;
/^[^+]/d;
s/^+//;
# s/\\\\/\\\\\\\\/g;
/^PATH=/ {
        s/\\\\/\\//g;
        s#\\([=;]\\)\\([a-zA-Z]\\):#\\1/cygdrive/\\2#g;
        s/;/:/g;
}
s/;/\\\\;/g;
s/^/export /;
s/=/='/;
s/\\\\r//g;
s/\r$/'/;
EOF
        /usr/bin/diff -u $srcenv $dstenv | /usr/bin/sed -f $envdiff >$diffenv
        . $diffenv
        PATH=$PATH:/bin:/usr/bin:/cygdrive/c/windows32/system32:/cygdrive/c/windows
        /bin/rm -f $srcenv $dstenv $diffenv $envdiff
}

for ARCH in ${ARCHS:-x86 x86_64}; do
	export ARCH

	(
		case "$ARCH" in
		x86)
       		export OSGEO4W_ROOT=/c/OSGeo4W
			export CMAKE_GEN="Visual Studio 14 2015"
       		export VCARCH=x86
			;;

		x86_64)
			export OSGEO4W_ROOT=/c/OSGeo4W64
        	export VCARCH=x64
			export CMAKE_GEN="Visual Studio 14 2015 Win64"
			;;

		*)
       		echo invalid arch
       		exit 1
			;;

		esac

		export R=$OSGEO4W_REP/$ARCH/release/$P
		mkdir -p $R

		cat <<EOF >$R/setup.hint
sdesc: "The PROJ library and commands for coordinate system transformations."
ldesc: "The PROJ library and commands for coordinate system transformations."
category: Libs Commandline_Utilities
requires: msvcrt2015 proj-hpgn proj-datumgrid proj-beta2007
maintainer: $MAINTAINER
EOF

		cp ../COPYING $R/$P-$V-$B.txt

		for i in proj-datumgrid-*.tar.gz; do
			p=${i%-*}
			v=${i%.tar.gz}
			v=${v##*-}

			! [ -f "$R/$p/$p-$v-1.tar.bz2" ] || continue

			mkdir -p tmp $R/$p
			/bin/tar -C tmp -xzf $i
			/bin/tar -C tmp -cjf $R/$p/$p-$v-1.tar.bz2 . --xform s,./,share/proj/,
			rm -r tmp

			cat <<EOF >$R/$p/setup.hint
sdesc: "Assorted PROJ datum grid shift files (${p#proj-datumgrid})"
ldesc: "Assorted PROJ datum grid shift files (${p#proj-datumgrid})"
category: Libs
requires:
maintainer: $MAINTAINER
EOF
		done

		fetchenv $OSGEO4W_ROOT/bin/o4w_env.bat
		fetchenv "/c/Program Files (x86)/Microsoft Visual Studio 14.0/VC/vcvarsall.bat" $VCARCH

		export PATH="/c/Program Files (x86)/Microsoft Visual Studio 14.0/Common7/IDE:/c/Program Files/CMake/bin:$PATH:/bin"
		export INSTDIR=$PWD/install-$ARCH
		export BUILDCONF=RelWithDebInfo

		rm -rf $INSTDIR
		mkdir -p $INSTDIR/etc/ini

		cat <<EOF >$INSTDIR/etc/ini/$P.bat
SET PROJ_LIB=%OSGEO4W_ROOT%\\share\\proj
EOF

		mkdir -p build-$ARCH
		cd build-$ARCH

		cmake -G "$CMAKE_GEN" \
			-DCMAKE_INSTALL_PREFIX=$(cygpath -m $INSTDIR) \
			-DPROJ_LIB_SUBDIR=lib \
			-DPROJ_CMAKE_SUBDIR=share/cmake/${P}4 \
			-DPROJ_DATA_SUBDIR=share/$P \
			-DPROJ_INCLUDE_SUBDIR=include \
			-DPROJ_TESTS=OFF \
			-DBUILD_LIBPROJ_SHARED=ON \
			-DBUILD_SHARED_LIBS=ON \
			../..

		for i in ZERO_CHECK ALL_BUILD INSTALL; do
			echo "$i: $(date)"
			cmake --build . --target $i --config $BUILDCONF
		done

		v=${V%.*}
		v=${v/./_}

		/bin/tar -C $INSTDIR -cjf $R/$P-$V-$B.tar.bz2 \
			--hard-dereference \
			--xform s,./lib/proj_$v.lib,lib/proj_i.lib, \
			bin \
			include \
			lib \
			./lib/proj_$v.lib \
			etc \
			share/proj \
			share/cmake/${P}4
		
		/bin/tar -C ../.. -cjf $R/$P-$V-$B-src.tar.bz2 \
			osgeo4w/package.sh \
			osgeo4w/0001-normalizeForVisualization-and-other-methods-applying.patch
	)
done
